Package: miaViz
Title: Microbiome Analysis Plotting and Visualization
Version: 1.21.0
Authors@R: 
    c(person(given = "Tuomas", family = "Borman", role = c("aut", "cre"),
             email = "tuomas.v.borman@utu.fi",
             comment = c(ORCID = "0000-0002-8563-8884")), 
      person(given = "Felix G.M.", family = "Ernst", role = c("aut"),
             comment = c(ORCID = "0000-0001-5064-0928")),
      person(given = "Leo", family = "Lahti", role = c("aut"),
             email = "leo.lahti@iki.fi",
             comment = c(ORCID = "0000-0001-5537-637X")),
      person(given = "Basil", family = "Courbayre", role = c("ctb")),
      person(given = "Giulio", family = "Benedetti", role = c("ctb"),
             comment = c(ORCID = "0000-0002-8732-7692")),
      person(given = "Théotime", family = "Pralas", role=c("ctb")),
      person(given = "Chouaib", family = "Benchraka", role=c("ctb")),
      person(given = "Sam", family = "Hillman", role=c("ctb")),
      person(given = "Geraldson", family = "Muluh", role=c("ctb")),
      person(given = "Noah", family = "De Gunst", role=c("ctb")),
      person(given = "Ely", family = "Seraidarian", role=c("ctb")),
      person(given = "Himmi", family = "Lindgren", role=c("ctb")),
      person(given = "Akewak", family = "Jeba", role=c("ctb"),
             comment = c(ORCID = "0009-0007-1347-7552")),
      person(given = "Vivian", family = "Ikeh", role=c("ctb")))
Description: The miaViz package implements functions to visualize
    TreeSummarizedExperiment objects especially in the context of microbiome
    analysis. Part of the mia family of R/Bioconductor packages.
biocViews: Microbiome, Software, Visualization
License: Artistic-2.0 | file LICENSE
Encoding: UTF-8
LazyData: false
Depends: R (>= 4.1.0), ggplot2, ggraph (>= 2.0), mia (>= 1.13.0),
        SummarizedExperiment, TreeSummarizedExperiment
Imports: ape, BiocGenerics, BiocParallel, DelayedArray,
        DirichletMultinomial, dplyr, ggnewscale, ggrepel, ggtree,
        methods, patchwork, rlang, S4Vectors, scales, scater,
        SingleCellExperiment, stats, tibble, tidygraph, tidyr,
        tidytext, tidytree, viridis
Suggests: beeswarm, BiocStyle, bluster, circlize, ComplexHeatmap,
        ggh4x, ggpubr, knitr, maaslin3, mediation, miaTime, patchwork,
        rmarkdown, rstatix, shadowtext, testthat, vegan, vipor
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
VignetteBuilder: knitr
URL: https://github.com/microbiome/miaViz
BugReports: https://github.com/microbiome/miaViz/issues
git_url: https://git.bioconductor.org/packages/miaViz
git_branch: devel
git_last_commit: 134960b
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:46:54 UTC; biocbuild
Author: Tuomas Borman [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8563-8884>),
  Felix G.M. Ernst [aut] (ORCID: <https://orcid.org/0000-0001-5064-0928>),
  Leo Lahti [aut] (ORCID: <https://orcid.org/0000-0001-5537-637X>),
  Basil Courbayre [ctb],
  Giulio Benedetti [ctb] (ORCID: <https://orcid.org/0000-0002-8732-7692>),
  Théotime Pralas [ctb],
  Chouaib Benchraka [ctb],
  Sam Hillman [ctb],
  Geraldson Muluh [ctb],
  Noah De Gunst [ctb],
  Ely Seraidarian [ctb],
  Himmi Lindgren [ctb],
  Akewak Jeba [ctb] (ORCID: <https://orcid.org/0009-0007-1347-7552>),
  Vivian Ikeh [ctb]
Maintainer: Tuomas Borman <tuomas.v.borman@utu.fi>
