# Generated by roxygen2: do not edit by hand

export(plotAbundanceDensity)
export(plotBarplot)
export(plotBoxplot)
export(plotCCA)
export(plotColGraph)
export(plotColTile)
export(plotDMNFit)
export(plotFeaturePrevalence)
export(plotForest)
export(plotHistogram)
export(plotLoadings)
export(plotMediation)
export(plotNMDS)
export(plotPrevalence)
export(plotPrevalentAbundance)
export(plotRDA)
export(plotRowGraph)
export(plotRowPrevalence)
export(plotRowTile)
export(plotScree)
export(plotSeries)
export(plotTaxaPrevalence)
exportMethods("colTreeData<-")
exportMethods("rowTreeData<-")
exportMethods(colTreeData)
exportMethods(combineTreeData)
exportMethods(getNeatOrder)
exportMethods(plotAbundance)
exportMethods(plotAbundanceDensity)
exportMethods(plotBarplot)
exportMethods(plotBoxplot)
exportMethods(plotCCA)
exportMethods(plotColGraph)
exportMethods(plotColTile)
exportMethods(plotColTree)
exportMethods(plotDMNFit)
exportMethods(plotFeaturePrevalence)
exportMethods(plotForest)
exportMethods(plotHistogram)
exportMethods(plotLoadings)
exportMethods(plotMediation)
exportMethods(plotPrevalence)
exportMethods(plotPrevalentAbundance)
exportMethods(plotRDA)
exportMethods(plotRowGraph)
exportMethods(plotRowPrevalence)
exportMethods(plotRowTile)
exportMethods(plotRowTree)
exportMethods(plotScree)
exportMethods(plotSeries)
exportMethods(plotTaxaPrevalence)
exportMethods(rowTreeData)
import(TreeSummarizedExperiment)
import(ggplot2)
import(ggraph)
import(methods)
import(mia)
importFrom(BiocGenerics,ncol)
importFrom(BiocGenerics,nrow)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bpisup)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpmapply)
importFrom(BiocParallel,bpstart)
importFrom(BiocParallel,bpstop)
importFrom(DelayedArray,rowMeans)
importFrom(DelayedArray,rowSums)
importFrom(DirichletMultinomial,mixture)
importFrom(S4Vectors,SimpleList)
importFrom(S4Vectors,metadata)
importFrom(S4Vectors,unfactor)
importFrom(SingleCellExperiment,reducedDim)
importFrom(SingleCellExperiment,reducedDimNames)
importFrom(SingleCellExperiment,reducedDims)
importFrom(SummarizedExperiment,"rowData<-")
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(TreeSummarizedExperiment,colTree)
importFrom(TreeSummarizedExperiment,colTreeNames)
importFrom(TreeSummarizedExperiment,rowTree)
importFrom(TreeSummarizedExperiment,rowTreeNames)
importFrom(ape,as.phylo)
importFrom(ape,drop.tip)
importFrom(ape,keep.tip)
importFrom(ape,rotateConstr)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,case_when)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,group_modify)
importFrom(dplyr,last_col)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,n_distinct)
importFrom(dplyr,pull)
importFrom(dplyr,relocate)
importFrom(dplyr,rename)
importFrom(dplyr,rename_with)
importFrom(dplyr,row_number)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,summarize)
importFrom(dplyr,ungroup)
importFrom(ggnewscale,new_scale)
importFrom(ggnewscale,new_scale_colour)
importFrom(ggnewscale,new_scale_fill)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotate)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expansion)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_errorbar)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_raster)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,labs)
importFrom(ggplot2,position_dodge)
importFrom(ggplot2,position_dodge2)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_fill_distiller)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_fill_viridis_d)
importFrom(ggplot2,scale_size_identity)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggrepel,geom_label_repel)
importFrom(ggrepel,geom_text_repel)
importFrom(ggtree,geom_cladelab)
importFrom(ggtree,geom_highlight)
importFrom(ggtree,geom_nodelab)
importFrom(ggtree,geom_nodepoint)
importFrom(ggtree,geom_tiplab)
importFrom(ggtree,geom_tippoint)
importFrom(ggtree,geom_tree)
importFrom(ggtree,ggtree)
importFrom(ggtree,gheatmap)
importFrom(ggtree,groupOTU)
importFrom(ggtree,theme_tree)
importFrom(mia,meltSE)
importFrom(patchwork,plot_layout)
importFrom(patchwork,wrap_plots)
importFrom(rlang,"!!")
importFrom(rlang,":=")
importFrom(rlang,sym)
importFrom(scales,pretty_breaks)
importFrom(scales,rescale)
importFrom(scater,plotReducedDim)
importFrom(scater,retrieveCellInfo)
importFrom(scater,retrieveFeatureInfo)
importFrom(stats,formula)
importFrom(stats,median)
importFrom(stats,qt)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(tibble,rownames_to_column)
importFrom(tidygraph,activate)
importFrom(tidygraph,as_tbl_graph)
importFrom(tidygraph,as_tibble)
importFrom(tidyr,complete)
importFrom(tidyr,drop_na)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidytext,reorder_within)
importFrom(tidytext,scale_y_reordered)
importFrom(tidytree,as.phylo)
importFrom(tidytree,as.treedata)
importFrom(tidytree,as_tibble)
importFrom(tidytree,child)
importFrom(tidytree,groupOTU)
importFrom(tidytree,isTip)
importFrom(tidytree,rootnode)
importFrom(utils,getFromNamespace)
importFrom(viridis,scale_colour_viridis)
importFrom(viridis,scale_fill_viridis)
