% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miaViz.R
\docType{package}
\name{miaViz-package}
\alias{miaViz}
\alias{miaViz-package}
\title{miaViz - Microbiome Analysis Plotting and Visualization}
\description{
The scope of this package is the plotting and visualization of microbiome
data. The main class for interfacing is the \code{TreeSummarizedExperiment}
class.
}
\seealso{
\link[mia:mia-package]{mia} class
}
\author{
\strong{Maintainer}: Tuomas Borman \email{tuomas.v.borman@utu.fi} (\href{https://orcid.org/0000-0002-8563-8884}{ORCID})

Authors:
\itemize{
  \item Felix G.M. Ernst (\href{https://orcid.org/0000-0001-5064-0928}{ORCID})
  \item Leo Lahti \email{leo.lahti@iki.fi} (\href{https://orcid.org/0000-0001-5537-637X}{ORCID})
}

Other contributors:
\itemize{
  \item Basil Courbayre [contributor]
  \item Giulio Benedetti (\href{https://orcid.org/0000-0002-8732-7692}{ORCID}) [contributor]
  \item Théotime Pralas [contributor]
  \item Chouaib Benchraka [contributor]
  \item Sam Hillman [contributor]
  \item Geraldson Muluh [contributor]
  \item Noah De Gunst [contributor]
  \item Ely Seraidarian [contributor]
  \item Himmi Lindgren [contributor]
  \item Akewak Jeba (\href{https://orcid.org/0009-0007-1347-7552}{ORCID}) [contributor]
  \item Vivian Ikeh [contributor]
}

}
