% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miaViz.R
\docType{data}
\name{mia-datasets}
\alias{mia-datasets}
\alias{col_graph}
\alias{row_graph}
\alias{row_graph_order}
\title{miaViz example data}
\format{
An object of class \code{tbl_graph} (inherits from \code{igraph}) of length 26.

An object of class \code{tbl_graph} (inherits from \code{igraph}) of length 996.

An object of class \code{tbl_graph} (inherits from \code{igraph}) of length 110.
}
\usage{
data(col_graph)

data(row_graph)

data(row_graph_order)
}
\description{
These example data objects were prepared to serve as examples. See the
details for more information.
}
\details{
For \code{*_graph} data:
\enumerate{
\item \dQuote{Jaccard} distances were calculated via
\code{calculateDistance(genus, FUN = vegan::vegdist, method = "jaccard",
exprs_values = "relabundance")}, either using transposed assay data or not
to calculate distances for samples or features. NOTE: the function
mia::calculateDistance is now deprecated.
\item \dQuote{Jaccard} dissimilarites were converted to similarities and values
above a threshold were used to construct a graph via
\code{graph.adjacency(mode = "lower", weighted = TRUE)}.
\item The \code{igraph} object was converted to \code{tbl_graph} via
\code{as_tbl_graph} from the \code{tidygraph} package.
}
}
\keyword{datasets}
