% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecate.R
\name{deprecate}
\alias{deprecate}
\alias{plotTaxaPrevalence}
\alias{plotTaxaPrevalence,ANY-method}
\alias{plotFeaturePrevalence}
\alias{plotFeaturePrevalence,ANY-method}
\title{These functions will be deprecated. Please use other functions instead.}
\usage{
plotTaxaPrevalence(x, ...)

\S4method{plotTaxaPrevalence}{ANY}(x, ...)

plotFeaturePrevalence(x, ...)

\S4method{plotFeaturePrevalence}{ANY}(x, ...)
}
\arguments{
\item{x}{\itemize{
\item 
}}

\item{...}{\itemize{
\item 
}}
}
\description{
These functions will be deprecated. Please use other functions instead.
}
